
//% color="#ff9f06" iconWidth=40 iconHeight=40
namespace pinpong_huskylens{

    //% block="读取文件[FNAME]" blockType="reporter"
    //% FNAME.shadow="string"  FNAME.defl="data.xlsx"
    export function readFile(parameter: any, block: any) {
        let fname = parameter.FNAME.code;
        let ftype = fname.slice(1,-1).split(".")[1];
        if (ftype == 'xlsx'){
            ftype = "excel";
        }

        Generator.addImport('import pandas as pd');
        Generator.addCode(`pd.read_${ftype}(${fname})`);
    }

    //% block="---"
    export function noteSep1() {

    }

    //% block="读取[DF]的索引" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    export function readIndex(parameter: any, block: any) {
        let df = parameter.DF.code;

        Generator.addCode(`${df}.index`);
    }

    //% block="读取[DF]的列标题" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    export function readColumns(parameter: any, block: any) {
        let df = parameter.DF.code;

        Generator.addCode(`${df}.columns`);
    }

    //% block="读取[DF]的数据" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    export function readValues(parameter: any, block: any) {
        let df = parameter.DF.code;

        Generator.addCode(`${df}.values`);
    }

    //% block="---"
    export function noteSep2() {

    }

    //% block="读取[DF][SIDE][NUM]行的数据" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    //% SIDE.shadow="dropdown"  SIDE.options="SIDE"
    //% NUM.shadow="range"   NUM.params.min=1      NUM.defl=5
    export function readRowValues(parameter: any, block: any) {
        let df = parameter.DF.code;
        let side = parameter.SIDE.code;
        let num = parameter.NUM.code;

        if (side == '最前'){
            side = "head";
        }else if (side == '最后'){
            side = "tail";
        }

        Generator.addCode(`${df}.${side}(${num})`);
    }

    //% block="读取[DF]的指定列[COL]的值" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    //% COL.shadow="string"  COL.defl="列标题"
    export function readOneColumnValues(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);

        Generator.addCode(`${df}["${col}"]`);
    }

    //% block="读取[DF]中单元格，行索引：[INDEX]，列标题：[COL]的值" blockType="reporter"
    //% DF.shadow="normal"  DF.defl="df"
    //% INDEX.shadow="number"  INDEX.defl=0
    //% COL.shadow="string"  COL.defl="列标题"
    export function readCell(parameter: any, block: any) {
        let df = parameter.DF.code;
        let index = parameter.INDEX.code;
        let col = replaceQuotationMarks(parameter.COL.code);

        Generator.addCode(`${df}.at[${index},"${col}"]`);
    }

    //% block="赋值[DF]中单元格，行索引：[INDEX]，列标题：[COL]的值为[VALUE]" blockType="command"
    //% DF.shadow="normal"  DF.defl="df"
    //% INDEX.shadow="number"  INDEX.defl=0
    //% COL.shadow="string"  COL.defl="列标题"
    //% VALUE.shadow="normal"  VALUE.defl="val"
    export function assignmentCell(parameter: any, block: any) {
        let df = parameter.DF.code;
        let index = parameter.INDEX.code;
        let col = replaceQuotationMarks(parameter.COL.code);
        let value = parameter.VALUE.code;

        Generator.addCode(`${df}.at[${index},"${col}"] = ${value}`);
    }


    //% block="批量赋值[DF]中[COL]列（若不存在则新增此列），值为[FORMULA]" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% COL.shadow="string"  COL.defl="列标题"
    //% FORMULA.shadow="normal"  FORMULA.defl='df["列标题"]*60+df["列标题"]'
    export function addCol(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);
        let formula = parameter.FORMULA.code;

        Generator.addCode(`${df}["${col}"] = ${formula}`);
    }

    //% block="---"
    export function noteSep3() {

    }

    //% block="删除[DF]中索引为[INDEX]的行" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% INDEX.shadow="number"  INDEX.defl=0
    export function dropRow(parameter: any, block: any) {
        let df = parameter.DF.code;
        let index = parameter.INDEX.code;

        Generator.addCode(`${df} = ${df}.drop(${index})`);
    }

    //% block="删除[DF]中列标题为[COL]的列" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% COL.shadow="string"  COL.defl="列标题"
    export function dropCol(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);

        Generator.addCode(`${df} = ${df}.drop("${col}",axis=1)`);
    }

    //% block="---"
    export function noteSep4() {

    }

    //% block="统计[DF]中指定列[COL]的[STAT]" blockType="reporter"
    //% DF.shadow="normal"  DF.defl=df
    //% COL.shadow="string"  COL.defl="列标题"
    //% STAT.shadow="dropdown"  STAT.options="STAT"
    export function statistics(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);
        let stat = parameter.STAT.code;

        if (stat == '非空项数量'){
            stat = "count";
        }else if (stat == '和'){
            stat = "sum";
        }else if (stat == '平均值'){
            stat = "mean";
        }else if (stat == '最大值'){
            stat = "max";
        }else if (stat == '最小值'){
            stat = "min";
        }

        Generator.addCode(`${df}["${col}"].${stat}()`);
    }

    //% block="统计[DF]中所有列的[STAT]" blockType="reporter"
    //% DF.shadow="normal"  DF.defl=df
    //% STAT.shadow="dropdown"  STAT.options="STAT"
    export function statisticsAll(parameter: any, block: any) {
        let df = parameter.DF.code;
        let stat = parameter.STAT.code;

        if (stat == '非空项数量'){
            stat = "count";
        }else if (stat == '和'){
            stat = "sum";
        }else if (stat == '平均值'){
            stat = "mean";
        }else if (stat == '最大值'){
            stat = "max";
        }else if (stat == '最小值'){
            stat = "min";
        }

        Generator.addCode(`${df}.${stat}()`);
    }



    //% block="---"
    export function noteSep5() {

    }

    //% block="筛选[DF]，条件为[CONDITION]" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% CONDITION.shadow="normal"  CONDITION.defl='df["列标题"]>=0'
    export function select(parameter: any, block: any) {
        let df = parameter.DF.code;
        let condition = parameter.CONDITION.code;

        Generator.addCode(`${df} = ${df}[ ${condition} ]`);
    }


    //% block="排序[DF]，按[COL]列 [ASCENDING]" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% COL.shadow="string"  COL.defl="列标题"
    //% ASCENDING.shadow="dropdown"  ASCENDING.options="ASCENDING"
    export function sort_values(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);
        let ascending = parameter.ASCENDING.code;

        if (ascending == '升序'){
            ascending = "True";
        }else{
            ascending = "False";
        }

        Generator.addCode(`${df} = ${df}.sort_values("${col}",ascending=${ascending})`);
    }


    //% block="分组[DF]，以[COL]列为分组依据，且[AS_INDEX]索引列" blockType="command"
    //% DF.shadow="normal"  DF.defl=df
    //% COL.shadow="string"  COL.defl="列标题"
    //% AS_INDEX.shadow="dropdown"  AS_INDEX.options="AS_INDEX"
    export function groupby(parameter: any, block: any) {
        let df = parameter.DF.code;
        let col = replaceQuotationMarks(parameter.COL.code);
        let as_index = parameter.AS_INDEX.code;

        if (as_index == '作为'){
            as_index = "True";
        }else{
            as_index = "False";
        }


        Generator.addCode(`${df} = ${df}.groupby("${col}",as_index=${as_index})`);
    }

    //% block="---"
    export function noteSep6() {

    }

    //% block="初始化pyplot" blockType="command"
    export function init_plt(parameter: any, block: any) {

        Generator.addImport('import matplotlib.pyplot as plt');
        Generator.addCode(`plt.rcParams['font.sans-serif'] = ['SimHei']`);
        Generator.addCode(`plt.figure()`);

    }


    //% block="绘制[KIND]，x轴为[X]，y轴为[Y]" blockType="command"
    //% KIND.shadow="dropdown"  KIND.options="KIND"
    //% X.shadow="normal"  X.defl='df["列标题"]'
    //% Y.shadow="normal"  Y.defl='df["列标题"]'
    export function draw(parameter: any, block: any) {
        var kind=parameter.KIND.code;
        var x=parameter.X.code;
        var y=parameter.Y.code;

        if (kind == '折线图'){
            kind = "plot";
        }else if (kind == '垂直柱形图'){
            kind = "bar";
        }else if (kind == '水平柱形图'){
            kind = "barh";
        }else if (kind == '散点图'){
            kind = "scatter";
        }

        Generator.addCode(`plt.${kind}(${x},${y})`);

    }

    //% block="设置标题[TITLE]" blockType="command"
    //% TITLE.shadow="string"  TITLE.defl="图表标题"
    export function title(parameter: any, block: any) {
        let title = replaceQuotationMarks(parameter.TITLE.code);

        Generator.addCode(`plt.title("${title}")`);

    }

    //% block="显示图例" blockType="command"
    export function legend(parameter: any, block: any) {

        Generator.addCode(`plt.legend()`);

    }

    //% block="显示绘图对象" blockType="command"
    export function show(parameter: any, block: any) {

        Generator.addCode(`plt.show()`);

    }

    //% block="---"
    export function noteSep7() {

    }



    function replaceQuotationMarks(str:string){
            str=str.replace(/"/g, ""); //去除所有引号
            return str
    }


    
}
